C===================================================================
C
      PROGRAM AEMLABE



C  This program solves the two-dimensinal General Linear 2nd
C  order Elliptic partial differential equation with variable 
C  coefficients using the Boundary Element Method with constant elements
C
C         A*uxx+2*B*uxy+C*uyy+D*ux+E*uy+F*u=f
C

   	USE MSIMSL
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)                     
      CHARACTER*15 INPUTFILE,OUTPUTFILE
C
C  Set the maximum dimensions
C
      PARAMETER (N=180) 
      PARAMETER (iN=103) 
C               
C  N=  Number of boundary elements equal to number of boundary 
C      nodes 
C  IN= Number of internal collocation points 
C
	COMMON C
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),XIN(IN),YIN(IN)
      DIMENSION C1(N,N),C2(N,N),C3(N)

      DIMENSION F(IN),COEF(IN,6),ICN(N+1)
	DIMENSION G(N,N),H(N,N) 
      DIMENSION UHAT(N,IN),QHAT(N,IN),T(N,IN)
      DIMENSION GIN(IN,N),GX(IN,N),GY(IN,N), 
     1          GXX(IN,N),GXY(IN,N),GYY(IN,N)
	DIMENSION HIN(IN,N),HX(IN,N),HY(IN,N),
     1          HXX(IN,N),HXY(IN,N),HYY(IN,N)
	DIMENSION UINHAT(IN,IN),UXHAT(IN,IN),UYHAT(IN,IN),
     1          UXXHAT(IN,IN),UXYHAT(IN,IN),UYYHAT(IN,IN)
 	DIMENSION Z(2*N,IN),E(2*N)
	DIMENSION S(IN,IN),SX(IN,IN),SY(IN,IN),SXX(IN,IN),SXY(IN,IN)
 	1         ,SYY(IN,IN)
	DIMENSION ZIN(IN),ZX(IN),ZY(IN),ZXX(IN),ZXY(IN),ZYY(IN)

	DIMENSION UB(N),QB(N)
      DIMENSION U(IN),UX(IN),UY(IN),UXX(IN),UXY(IN),UYY(IN)

	DIMENSION D(N,N)
C	
C
C  XL,YL     = Coordinates of the extreme points of the boundary elements
C  XM,YM     = Coordinates of the midpoints of the boundary elements
C  XIN,YIN   = Coordinates of the internal collocation points 
C  UB,QB     = Boundary nodal values of u and un
C  G,H       = Influence matrices Gij and Hij-ij relating the boundary points
C  UHAT,QHAT = Influence matrices resulting from the particular solution Uhat
C           	 and its normal derivative Qhat relating the internal 
C              points and the boundary nodal points
C
C  GIN,GX,GY,GINXX,GXY,GYY 
C		   = Influence matrx Gij and its derivatives relating 
C              the internal points and the boundary nodal points
C
C  HIN,HX,HY,HX,HXY,HYY 
C            = Influence matrix Hij and its derivatives relating 
C              the internal points and the boundary nodal points
C
C  UINHAT,UXHAT,UYHAT,UXXHAT,UXYHAT,UYYHAT
C            = Influence matrix Uij and its derivatives relating 
C              the internal points 
C
C  U,UX,UY,UXX,UXY,UYY
C  		   =  Solution vector and its derivatives at th IN internal points
C
C  f         = Right hand side load vector
C  c1,c2,c3  = MATRICES & Vectors including the values of the coefficients
C              of the boundary conditions 
C  COEF      = INx6 array including the coefficients A,B,C,D,E,F of the equation
C              at the IN internal points

C	   
C  Read the names and open the input and output files
C   

	WRITE (*,'(A)')' Name of the INPUTFILE (max.15 characters)'
      READ  (*,'(A)') INPUTFILE
      WRITE (*,'(A)')' Name of the OUTPUTFILE (max.15 characters)'
      READ  (*,'(A)') OUTPUTFILE
      OPEN (UNIT=1, FILE=inputfile)
      OPEN (UNIT=2, FILE=OUTputfile)	  

	READ (*,*) C
C
C  Read data from INPUTFILE
C
	CALL INPUT (XL,YL,XIN,YIN,C1,C2,C3,F,COEF,N,IN,ICN,D) 

      WRITE(*,*)('FFFFF 1 FFFFF')
C
C  Compute the G matrix
C
      CALL GMATR (XL,YL,XM,YM,G,N) 

      WRITE(*,*)('FFFFF 2 FFFFF')
C         
C  Compute the H matrix
C
      CALL HMATR (XL,YL,XM,YM,H,N)
C
      WRITE(*,*)('FFFFF 3 FFFFF')
C
C  Compute the  matrices UHAT,QHAT and T
C
      CALL UQTMATR (XL,YL,XM,YM,XIN,YIN,G,H,UHAT,QHAT,T,N,IN)

      WRITE(*,*)('FFFFF 4 FFFFF')
C
C  Compute the matrix Z and the vector e 
C
      CALL BOUNDMATR(G,H,T,C1,C2,C3,N,IN,Z,E) 

      WRITE(*,*)('FFFFF 5 FFFFF')
C
C  Compute the matrices GIN, Gx,Gy,...,Hin,..Hyy
C
      CALL GHINMATR(XL,YL,XIN,YIN,GIN,GX,GY,GXX,GXY,GYY,
     1	                   HIN,HX,HY,HXX,HXY,HYY,N,IN)

      WRITE(*,*)('FFFFF 6 FFFFF')
C
C  Compute the  matrices UHATIN,UHATx,UHATy,....,UHATyy
C
      CALL UINHATMATR(XL,YL,XIN,YIN,UINHAT,UXHAT,UYHAT,
     1	                      UXXHAT,UXYHAT,UYYHAT,N,IN)

      WRITE(*,*)('FFFFF 7 FFFFF')
C
C  Compute the  matrices S, Sx, Sy, Sxx, Sxy, Syy and the 
C  vectors z,zx,zy,zxx,zxy,zyy
C
      CALL SZMATR(GIN,GX,GY,GXX,GXY,GYY,HIN,HX,HY,HXX,HXY,HYY,UINHAT
	1    ,UXHAT,UYHAT,UXXHAT,UXYHAT,UYYHAT,UHAT,QHAT,Z,E,N,IN
     1    ,S,SX,SY,SXX,SXY,SYY,ZIN,ZX,ZY,ZXX,ZXY,ZYY)
 	     
      WRITE(*,*)('FFFFF 8 FFFFF')
C
C  Compute the values of the solution U,Ux,Uy,Uxx,Uxy,Uyy at the 
C  internal points and UB,QB at boundary points

	CALL SOLVE(Z,E,S,SX,SY,SXX,SXY,SYY,ZIN,ZX,ZY,ZXX,ZXY,ZYY,COEF,F
	1        ,N,IN,UB,QB,U,UX,UY,UXX,UXY,UYY)

      WRITE(*,*)('FFFFF  9  FFFFF')

C
C  Print the results in the OUTPUTFILE
C
      CALL OUTPUT(XM,YM,UB,QB,XIN,YIN,U,UX,UY,UXX,UYY,UXY,N,IN,D)
C
      WRITE(*,*)('FFFFFF 10 FFFFF')
C
C  Close input and output files
C
      CLOSE(1)
      CLOSE(2)
      STOP
      END
C      
C====================================================================
C
      SUBROUTINE INPUT (XL,YL,XIN,YIN,C1,C2,C3,F,COEF,N,IN,ICN,D)   
C     
C  This subroutine reads the input data from the input file
C  and writes them in the output file 
C
      IMPLICIT  DOUBLEPRECISION (A-H,O-Z)
      CHARACTER*80 NAME,TITLE
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN),ICN(n+1)
      DIMENSION B1(N,N),B2(N,N),C3(N),F(IN),COEF(IN,6)
	DIMENSION S1(N,N),S2(N,N),C1(N,N),C2(N,N),D(N,N),COEFB(N,3)

      WRITE(2,100)
  100 FORMAT(' ',79('*'))
C
C  Read user's name
C
      READ(1,'(A)')NAME
      WRITE(2,'(A)')NAME
C
C  Read the title of the program
C
      READ(1,'(A)')TITLE
      WRITE(2,'(A)')TITLE
      WRITE(2,200)N,IN
  200 FORMAT(//'BASIC PARAMETERS'//2X,'NUMBER OF BOUNDARY ELEMENTS='
     1,I4/2X,'NUMBER OF INTERNAL COLLOCATION POINTS =',I5)
C
C  Read the coordinates of the extreme points of the boundary elements XL,YL
C
      READ(1,*) (XL(I),YL(I),icn(i),I=1,N)
	xl(n+1)=xl(1)
	yl(n+1)=yl(1)
	icn(n+1)=icn(1)
C
C  Write the coordinates in the output file
C
      WRITE(2,300)
  300 FORMAT(//2X,'COORDINATES OF THE EXTREME POINTS OF THE BOUNDARY ELE
     1MENTS',//2X,'POINT',9X,'XL',15X,'YL')
      DO 20 I=1,N
   20 WRITE(2,400) I,XL(I),YL(I)
  400 FORMAT(2X,I3,2(3X,E14.5))
C      
C  Read the coefficients c1,c2 and c3 of the boundary conditions Eq.(8.81)
C  .
      READ(1,*) (B1(I,I),B2(I,I),C3(I), I=1,N)
C
C  Write the coefficients of the boundary conditions in the output file
C
      WRITE(2,500)
  500 FORMAT(//2X,'COEFFICIENTS OF THE BOUNDARY CONDITIONS'//2X,'NODE',
     # 15X,'c1',12X, 'c2',12X, 'c3')
      DO 30 I=1,N
   30 WRITE(2,600) I,B1(I,I),B2(I,I),C3(I)
  600 FORMAT (2X,I3,8X,3E14.5)
C
C  Read the coordinates of the internal points
C
      READ(1,*) (XIN(I),YIN(I),I=1,IN)
C
C  Read the vector f including the values of the right hand 
C  side of the equation 
C     
      READ(1,*) (F(I),I=1,IN)
C      
C Write XIN, YIN and f in the output file      
C
      WRITE(2,700)
  700 FORMAT(//2X,'RIGHT HAND SIDE OF THE EQUATION'//
     1         9X,'XIN',8X,'YIN',12X,'f')
      DO 40 I=1,IN
   40 WRITE(2,800)XIN(I),YIN(I),F(I)
  800 FORMAT(2X,2(F10.2,2X),E14.5)
C
C Read the array of the coefficients of the equation at internal
C points and write them in the outputfile 
C
      DO 50 I=1,IN
	READ(1,*) (COEF(I,J),J=1,6)
   50 CONTINUE
  610 FORMAT(6(F16.8))
   	WRITE(2,900)
  900	FORMAT(//2X,'COEFFICIENTS OF THE EQUATION AT INTERNAL POINTS'//)
	DO 70 I =1,IN
   70	WRITE(2,610) (COEF(I,J),J=1,6)
C
C Read the array of the coefficients of the equation at boundary
C points, write them in the outputfile   
C
	C1=0.
	C2=0.
	S1=0.
	S2=0.
   	WRITE(2,950)
	DO 60 I=1,N
	READ(1,*) (COEFB(I,J),J=1,3)
	WRITE (2,610) (COEFB(I,J),J=1,3)
      DX=XL(I+1)-XL(I)    
      DY=YL(I+1)-YL(I)
      SL=DSQRT(DX**2+DY**2)
      ENX=DY/SL
      ENY=-DX/SL
	A=COEFB(I,1)
	B=COEFB(I,2)
	C=COEFB(I,3)
	S1(I,I)=A*ENX**2+2.*B*ENX*ENY+C*ENY**2
	S2(I,I)=B*(ENX**2-ENY**2)+(C-A)*ENX*ENY
   60 CONTINUE
  950	FORMAT(//2X,'COEFFICIENTS OF THE EQUATION AT BOUNDARY POINTS'//)
C
C Compute the finite difference matrix D of Eq. (8.83)
C
	D=0.
	CALL FINDIF(XL,YL,ICN,N,D)
C
C Compute the matrices C1,C2 of Eq.(8.86)  
C
	S2=MATMUL(S2,D)
	C2=MATMUL(B2,S1)
	C1=B1+MATMUL(B2,S2)

      RETURN
      END
C      
C      
C==================================================================
C
      SUBROUTINE GMATR (XL,YL,XM,YM,G,N)
C
C  This subroutine computes the elements of the G matrix
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N)
      DIMENSION G(N,N)
C
C  Compute the boundary nodal coordinates and store them in  
C  the arrays XM and YM
C
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
      DO 10  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
   10 YM(I)=(YL(I)+YL(I+1))/2.
C
C  Compute the elements of matrix G
C
      DO 20 I=1,N
      DO 20 J=1,N
      JP1=J+1
      IF(I.NE.J)THEN
      CALL RLINTC(XM(I),YM(I),XL(J),YL(J),XL(JP1),YL(JP1),RESULT)     
      G(I,J)=RESULT
      ELSEIF(I.EQ.J)THEN
      CALL SLINTC(XL(J),YL(J),XL(JP1),YL(JP1),RESULT)
      G(I,J)=RESULT
      ENDIF
   20 CONTINUE
      RETURN
      END
C
C
C================================================================
C
      SUBROUTINE RLINTC(X0,Y0,X1,Y1,X2,Y2,RESULT)
C
C
C  This subroutine computes the off-diagonal elelments of the 
C  matrix G 
C
C  RA= The distance of the point O from the Gauss integration point 
C      on the boundary element
C
C  WG= The weights of the Gauss integration
C
C  XI= The coordinates of the Gauss integration points in the 
C      interval [-1,1] 
C
C  XC,YC=The global coordinates of the Gauss integration points
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      DIMENSION XC(4),YC(4),XI(4),WG(4)
      DATA XI/-0.86113631,-0.33998104,0.33998104,0.86113631/
      DATA WG/0.34785485,0.65214515,0.65214515,0.34785485/
      PI=ACOS(-1.)
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      BX=(X2+X1)/2.
      BY=(Y2+Y1)/2.
C
C  Compute the line integral 
C
      RESULT=0.
C                  
      DO 30  I=1,4
      XC(I)=AX*XI(I)+BX
      YC(I)=AY*XI(I)+BY
      RA=DSQRT((XC(I)-X0)**2+(YC(I)-Y0)**2)
   30 RESULT=RESULT+DLOG(RA)*WG(I)
      SL=2.*DSQRT(AX**2+AY**2)
      RESULT=RESULT*SL/(4.*PI)
      RETURN
      END
C
C
C=================================================================
C
      SUBROUTINE SLINTC(X1,Y1,X2,Y2,RESULT) 
C
C
C  This subroutines computes the diagonal elements of the matrix G
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      PI=ACOS(-1.)
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      SL=DSQRT(AX**2+AY**2)
      RESULT=SL*(DLOG(SL)-1.)/PI
      RETURN
      END
C
C
C=================================================================
C												
      SUBROUTINE HMATR(XL,YL,XM,YM,H,N)
C
C
C  This subroutine computes the element of the H matrix
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)     
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N)
      DIMENSION H(N,N)
C
      PI=ACOS(-1.)
C
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
      DO 10  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
   10 YM(I)=(YL(I)+YL(I+1))/2.

C
C  Compute the elements of H matrix
C
      DO 20  I=1,N
      DO 20  J=1,N
      IF (I.NE.J) THEN      
      CALL DALPHA (XM(I),YM(I),XL(J),YL(J),XL(J+1),YL(J+1),RESULT)
      H(I,J)=RESULT
      ELSEIF (I.EQ.J) THEN
      H(I,J)=-0.5
      ENDIF
   20 CONTINUE
      RETURN
      END
C
C
C=================================================================
C
      SUBROUTINE DALPHA(X0,Y0,X1,Y1,X2,Y2,RESULT)
C
C
C  This subroutine computes the off diagonal elements of the 
C  matrix H 
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      PI=ACOS(-1.)
      DY1=Y1-Y0
      DX1=X1-X0
      DY2=Y2-Y0
      DX2=X2-X0
      DL1=DSQRT(DX1**2+DY1**2)
      COS1=DX1/DL1
      SIN1=DY1/DL1
      DX2R=DX2*COS1+DY2*SIN1
      DY2R=-DX2*SIN1+DY2*COS1
      DA=ATAN2(DY2R,DX2R)
      RESULT=DA/(2.*PI)
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE UQTMATR (XL,YL,XM,YM,XIN,YIN,G,H,UHAT,QHAT,T,N,IN)
C
C
C  This subroutine computes the matrices UHAT,QHAT and T
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
	COMMON C
      DIMENSION G(N,N),H(N,N),UHAT(N,IN),QHAT(N,IN),T(N,IN)
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),XIN(IN),YIN(IN)
	
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
      DO 5  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
    5 YM(I)=(YL(I)+YL(I+1))/2.
C
      WRITE(2,*)
	WRITE (*,*) 'MQs constant C :',C
	WRITE (2,*) 'MQs constant C :',C
C      
      DO 10 I=1,N
      DO 10 J=1,IN
      R=DSQRT((XM(I)-XIN(J))**2+(YM(I)-YIN(J))**2)
C
C  Computation of UHAT
C      
      UHAT(I,J)=-C**3/3.*DLOG(C*DSQRT(R**2+C**2)+C**2)+
	1               (R**2+4.*C**2)*DSQRT(R**2+C**2)/9.
C
C  Computation of the direction cosines ofthe normal unit vector n
C
         DX=XL(I+1)-XL(I)    
         DY=YL(I+1)-YL(I)
         SL=DSQRT(DX**2+DY**2)
         ENX=DY/SL
         ENY=-DX/SL
C
C  Computation of QHAT
C
      QHAT(I,J)=1./(3.*DSQRT(R**2+C**2))*(R**2+2.*C**2-C**3/
	1  (DSQRT(R**2+C**2)+C))*((XM(I)-XIN(J))*ENX+(YM(I)-YIN(J))*ENY)
   10 CONTINUE
C
C  Computation of T
C
	T=0.
	T=MATMUL(H,UHAT)-MATMUL(G,QHAT)

      RETURN
      END

C
C
C==================================================================
C
	SUBROUTINE BOUNDMATR(G,H,T,C1,C2,C3,N,IN,Z,E)
C
C  This subroutine computes the matrices Z, e
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)

      DIMENSION G(N,N),H(N,N),T(N,IN),C1(N,N),C2(N,N),C3(N)
	DIMENSION Z(2*N,IN),E(2*N)
	DIMENSION A(2*N,2*N),AI(2*N,2*N)

	A=0.
	DO 10 I=1,N
	DO 10 J=1,N
	A(I,J)=H(I,J)
	A(I,J+N)=-G(I,J)
	A(I+N,J)=C1(I,J)
	A(I+N,J+N)=C2(I,J)
   10 CONTINUE

	Z=0.
	E=0.
	DO 20 I=1,N
	E(I+N)=C3(I)
	DO 20 J=1,IN
	Z(I,J)=T(I,J)
	
   20 CONTINUE

	AI=0.
	CALL DLINRG(2*N,A,2*N,AI,2*N)


	Z=MATMUL(AI,Z)
	E=MATMUL(AI,E)

	RETURN
	END
C
C===================================================================
C
      SUBROUTINE GHINMATR (XL,YL,XIN,YIN,GIN,GX,GY,GXX,GXY,GYY,
     1	                           HIN,HX,HY,HXX,HXY,HYY,N,IN)
C
C  This subroutine computes the INxN matrices Gin, Gx,Gy,...,Hin,..Hyy
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
 	COMMON C
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN)
      DIMENSION GIN(IN,N),GX(IN,N),GY(IN,N),GXX(IN,N),
     1          GXY(IN,N),GYY(IN,N)
      DIMENSION HIN(IN,N),HX(IN,N),HY(IN,N),HXX(IN,N),
     1          HXY(IN,N),HYY(IN,N)
      DIMENSION XC(4),YC(4),XI(4),WG(4)
C
C  WG       =Weights of the Gauss integration
C  XI       =Co-ordinates of of the Gauss integration points
C             in the interval [-1,1]
C  XC,YC    =Global co-ordinates of the gauss integration points
C
      DATA XI/-0.86113631,-0.33998104,0.33998104,0.86113631/
      DATA WG/0.34785485,0.65214515,0.65214515,0.34785485/
      PI=ACOS(-1.)
      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
C
      DO 10 I=1,IN
      DO 10 J=1,N
C         
C  Computation of the direction cosines nx,ny of the normal
C  unit vector n on the J boundary element          
C
      AX=(XL(J+1)-XL(J))/2.
      AY=(YL(J+1)-YL(J))/2.
      BX=(XL(J+1)+XL(J))/2.
      BY=(YL(J+1)+YL(J))/2.
      SL=DSQRT(AX**2+AY**2)
      ENX=AY/SL
      ENY=-AX/SL
C      
C Computation Gin, Gx,Gy,...,Hin,..Hyy  
C
         U=0.
         UX=0.
	   UY=0.
	   UXX=0.
	   UXY=0.
	   UYY=0.
         UN=0.
         UNX=0.
	   UNY=0.
	   UNXX=0.
	   UNXY=0.
	   UNYY=0.

      DO 40 IK=1,4
        XC(IK)=AX*XI(IK)+BX
        YC(IK)=AY*XI(IK)+BY
        DX=XC(IK)-XIN(I)
        DY=YC(IK)-YIN(I)
        R=DSQRT(DX**2+DY**2)
        RX=-DX/R
        RY=-DY/R
	  S=DLOG(R)/(2.*PI)	  
        SX=RX/(2.*PI*R)
        SY=RY/(2.*PI*R)
	  SXX=2.*PI*(SY**2-SX**2)
	  SYY=-SXX
	  SXY=-4.*PI*SX*SY

	  RN=-(RX*ENX+RY*ENY)
        RT=-(-RX*ENY+RY*ENX)

	  SN=RN/(2.*PI*R)
	  ST=RT/(2.*PI*R)
	  SNX=2.*PI*(SY*ST-SX*SN)
	  SNY=-2.*PI*(SX*ST+SY*SN)
	  SNXX=4.*PI*(SY*SNY-SX*SNX)
	  SNYY=-SNXX
	  SNXY=-4.*PI*(SY*SNX+SX*SNY)
C
        U=U+S*WG(IK)*SL
        UX=UX+SX*WG(IK)*SL
        UY=UY+SY*WG(IK)*SL
        UXX=UXX+SXX*WG(IK)*SL
        UXY=UXY+SXY*WG(IK)*SL
	  UYY=UYY+SYY*WG(IK)*SL
        UN=UN+SN*WG(IK)*SL
        UNX=UNX+SNX*WG(IK)*SL
	  UNY=UNY+SNY*WG(IK)*SL
	  UNXX=UNXX+SNXX*WG(IK)*SL
	  UNYY=UNYY+SNYY*WG(IK)*SL
	  UNXY=UNXY+SNXY*WG(IK)*SL
   40 CONTINUE
C
         GIN(I,J)=U
         GX(I,J)=UX
	   GY(I,J)=UY
	   GXX(I,J)=UXX
	   GXY(I,J)=UXY
	   GYY(I,J)=UYY
         HIN(I,J)=UN
         HX(I,J)=UNX
	   HY(I,J)=UNY
	   HXX(I,J)=UNXX
	   HXY(I,J)=UNXY
	   HYY(I,J)=UNYY
C
   10 CONTINUE
	RETURN
	END
C
C==================================================================
C
      SUBROUTINE UINHATMATR(XL,YL,XIN,YIN,UINHAT,UXHAT,UYHAT,
     1	                      UXXHAT,UXYHAT,UYYHAT,N,IN)
C
C  This subroutine computes the values of the matrices UHTIN,UHATX,
C  UHATY, UHATXX, UHATXY, UHATYY
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
	COMMON C
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN)
      DIMENSION UINHAT(IN,IN),UXHAT(IN,IN),UYHAT(IN,IN)
	DIMENSION UXXHAT(IN,IN),UXYHAT(IN,IN),UYYHAT(IN,IN)

      XL(N+1)=XL(1)
      YL(N+1)=YL(1)
C
      DO 10 I=1,IN
	DO 10 J=1,IN
C
C   Computation of UINHAT
C
      R=DSQRT((XIN(I)-XIN(J))**2+(YIN(I)-YIN(J))**2)
      UINHAT(I,J)=-C**3/3.*DLOG(C*DSQRT(R**2+C**2)+C**2)+
	1        (R**2+4.*C**2)*DSQRT(R**2+C**2)/9.
C   
C   Computation of UXHAT,UYHAT
C   
      UXHAT(I,J)=1./3./DSQRT(R**2+C**2)*(R**2+2.*C**2-C**3/
	1                          (DSQRT(R**2+C**2)+C))*(XIN(I)-XIN(J))
      UYHAT(I,J)=1./3./DSQRT(R**2+C**2)*(R**2+2.*C**2-C**3/
	1                          (DSQRT(R**2+C**2)+C))*(YIN(I)-YIN(J))
C
C   Computation of UXXHAT, UXYHAT, UYYHAT
C     
      IF(I.EQ.J) THEN
         UXXHAT(I,J)=C/2.
         UYYHAT(I,J)=C/2.
         UXYHAT(I,J)=0.
      ELSEIF(I.NE.J) THEN
	U1=1./3./(R**2+C**2)**(3./2.)*(R**2+C**3*(2.*DSQRT(R**2+C**2)+C)/
	1	  (DSQRT(R**2+C**2)+C)**2)
	U2=1./3./DSQRT(R**2+C**2)*(R**2+2.*C**2-C**3/
     1	  (DSQRT(R**2+C**2)+C))

         UXXHAT(I,J)=U1*(XIN(I)-XIN(J))**2+U2
         UYYHAT(I,J)=U1*(YIN(I)-YIN(J))**2+U2
         UXYHAT(I,J)=U1*(XIN(I)-XIN(J))*(YIN(I)-YIN(J))
      ENDIF
C
   10 CONTINUE   
      RETURN
      END
C
C=================================================================
C
	SUBROUTINE SZMATR(GIN,GX,GY,GXX,GXY,GYY,HIN,HX,HY,HXX,HXY,HYY
	1   ,UINHAT,UXHAT,UYHAT,UXXHAT,UXYHAT,UYYHAT,UHAT,QHAT,Z,E,N,IN
     1   ,S,SX,SY,SXX,SXY,SYY,ZIN,ZX,ZY,ZXX,ZXY,ZYY)
C
C  This subroutine computes the  matrices S, Sx, Sy, Sxx, Sxy, Syy 
C  and the vectors z,zx,zy,zxx,zxy,zyy
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)

      DIMENSION UHAT(N,IN),QHAT(N,IN)
      DIMENSION GIN(IN,N),GX(IN,N),GY(IN,N), 
     1          GXX(IN,N),GXY(IN,N),GYY(IN,N)
	DIMENSION HIN(IN,N),HX(IN,N),HY(IN,N),
     1          HXX(IN,N),HXY(IN,N),HYY(IN,N)
	DIMENSION UINHAT(IN,IN),UXHAT(IN,IN),UYHAT(IN,IN),
     1          UXXHAT(IN,IN),UXYHAT(IN,IN),UYYHAT(IN,IN)
	DIMENSION Z(2*N,IN),E(2*N)
	DIMENSION S(IN,IN),SX(IN,IN),SY(IN,IN),SXX(IN,IN),SXY(IN,IN)
	1         ,SYY(IN,IN)
	DIMENSION ZIN(IN),ZX(IN),ZY(IN),ZXX(IN),ZXY(IN),ZYY(IN)

	S=0.
	S=UINHAT+MATMUL(GIN,QHAT)-MATMUL(HIN,UHAT)
	SX=0.
	SX=UXHAT+MATMUL(GX,QHAT)-MATMUL(HX,UHAT) 
	SY=0.
	SY=UYHAT+MATMUL(GY,QHAT)-MATMUL(HY,UHAT)
	SXX=0.
	SXX=UXXHAT+MATMUL(GXX,QHAT)-MATMUL(HXX,UHAT)
	SXY=0.
	SXY=UXYHAT+MATMUL(GXY,QHAT)-MATMUL(HXY,UHAT)
	SYY=0.
	SYY=UYYHAT+MATMUL(GYY,QHAT)-MATMUL(HYY,UHAT)

	ZIN=0.
	ZX=0.
	ZY=0.
	ZXX=0.
	ZXY=0.
	ZYY=0.

	DO 10 I=1,IN
	DO 10 K=1,IN
 	DO 10 J=1,N
	S(I,K)=S(I,K)    + HIN(I,J)*Z(J,K) - GIN(I,J)*Z(J+N,K)
	SX(I,K)=SX(I,K)  + HX(I,J)*Z(J,K)  - GX(I,J)*Z(J+N,K)
	SY(I,K)=SY(I,K)  + HY(I,J)*Z(J,K)  - GY(I,J)*Z(J+N,K)
	SXX(I,K)=SXX(I,K)+ HXX(I,J)*Z(J,K) - GXX(I,J)*Z(J+N,K)
	SXY(I,K)=SXY(I,K)+ HXY(I,J)*Z(J,K) - GXY(I,J)*Z(J+N,K)
	SYY(I,K)=SYY(I,K)+ HYY(I,J)*Z(J,K) - GYY(I,J)*Z(J+N,K)
   10	CONTINUE 

	DO 20 I=1,IN
	DO 20 J=1,N
	ZIN(I)=ZIN(I)+ HIN(I,J)*E(J)-GIN(I,J)*E(J+N)
	ZX(I) =ZX(I) + HX(I,J)*E(J) -GX(I,J)*E(J+N)
	ZY(I) =ZY(I) + HY(I,J)*E(J)	-GY(I,J)*E(J+N)
	ZXX(I)=ZXX(I)+ HXX(I,J)*E(J)-GXX(I,J)*E(J+N)
	ZXY(I)=ZXY(I)+ HXY(I,J)*E(J)-GXY(I,J)*E(J+N)
	ZYY(I)=ZYY(I)+ HYY(I,J)*E(J)-GYY(I,J)*E(J+N)
   20	CONTINUE
	RETURN
	END
C
C==================================================================
C
	SUBROUTINE SOLVE(Z,E,S,SX,SY,SXX,SXY,SYY,ZIN,ZX,ZY,ZXX,ZXY,ZYY
	1                  ,COEF,F,N,IN,Ub,Qb,U,UX,UY,UXX,UXY,UYY)
C
C  This subroutine solves the system of equations	K*A=P and computes the 
C  solution U,Ux,Uy,Uxx,Uyy,Uxy at internal points and Ub,Qb at boundary
C  points
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
	DIMENSION Z(2*N,IN),E(2*N),F(N),COEF(IN,6),P(IN),A(IN)
	DIMENSION S(IN,IN),SX(IN,IN),SY(IN,IN),SXX(IN,IN),SXY(IN,IN)
	1         ,SYY(IN,IN)
	DIMENSION ZIN(IN),ZX(IN),ZY(IN),ZXX(IN),ZXY(IN),ZYY(IN)
      DIMENSION U(IN),UX(IN),UY(IN),UXX(IN),UXY(IN),UYY(IN)
	DIMENSION Ub(N),Qb(N)
	DOUBLEPRECISION K(IN,IN)

C
C  Formulate system of equations
C
	K=0.
	P=0.
	DO 10 I=1,IN
	P(I)=F(I)-(COEF(I,1)*ZXX(I) + 2.*COEF(I,2)*ZXY(I) + 
 	1       COEF(I,3)*ZYY(I) +    COEF(I,4)*ZX(I)  +
     1       COEF(I,5)*ZY(I)  +    COEF(I,6)*ZIN(I))

	DO 10 J=1,IN

	K(I,J)= COEF(I,1)*SXX(I,J) + 2.*COEF(I,2)*SXY(I,J) + 
	1       COEF(I,3)*SYY(I,J) +    COEF(I,4)*SX(I,J)  +
     1       COEF(I,5)*SY(I,J)  +    COEF(I,6)*S(I,J)
     
   10 CONTINUE
C
C  Solve linear system of equations K*X=P
C
      CALL LEQS(K,P,IN,LSING)
      IF(LSING.EQ.0)THEN
      WRITE(2,150)
  150 FORMAT(/,'',69('*')//2X'The system has been solved regularly'/)
      ELSEIF(LSING.EQ.1)THEN
      WRITE(2,170)
  170 FORMAT(/,'',69('*')//2X'The system is singular'/)
      ENDIF
	A=P
C
C  Compute u,ux,uy,uxx,uyy and uxy at the internal points
C
	U=0.
	U=MATMUL(S,A)+ZIN
	UX=0.
	UX=MATMUL(SX,A)+ZX
	UY=0.
	UY=MATMUL(SY,A)+ZY
	UXX=0.
	UXX=MATMUL(SXX,A)+ZXX
	UXY=0.
	UXY=MATMUL(SXY,A)+ZXY
	UYY=0.
	UYY=MATMUL(SYY,A)+ZYY
C
C  Compute Ub,Qb at boundary points
C   
	Ub=0.
	Qb=0.
	DO 20 I=1,N
	Ub(I)=E(I)
	Qb(I)=E(I+N)
	DO 20 J=1,IN
	Ub(I)=Ub(I)+Z(I,J)*A(J)
	Qb(I)=Qb(I)+Z(I+N,J)*A(J)
   20	CONTINUE
	RETURN
	END

C
C==================================================================
C
      SUBROUTINE LEQS(A,B,N,LSING)
C
C   This subroutine uses Gauss elimination to solve
C   a system of linear equations, [A]{X}={B}, where
C   A : One-dimensional array which contains the occasional row of
C       the two-dimensional array of the coefficients of the unknowns
C   B : One-dimensional array which contains the known coefficients
C   N : Integer denoting the number of the unknowns
C   LSING: Integer taking the values:
C         LSING = 0, if the system has been solved regularly
C         LSING = 1, if the system is singular
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(1)
C
      LSING=0
      JJ=-N
      DO 10 J=1,N
        JY=J+1
        JJ=JJ+N+1
        AMAX=0.0
        IHELP=JJ-J
      DO 20 I=J,N
        IJ=IHELP+I
        IF(ABS(AMAX)-ABS(A(IJ))) 30,20,20
   30      AMAX=A(IJ)
           IMAX=I
   20 CONTINUE
         IF(ABS(AMAX).EQ.0.) THEN
            LSING=1
            RETURN
         END IF
      I1=J+N*(J-2)
      IHELP=IMAX-J
      DO 40 K=J,N
        I1=I1+N
        I2=I1+IHELP
        ATEMP=A(I1)
        A(I1)=A(I2)
        A(I2)=ATEMP
   40 A(I1)=A(I1)/AMAX
      ATEMP=B(IMAX)
      B(IMAX)=B(J)
      B(J)=ATEMP/AMAX
         IF(J-N) 50,70,50
   50       IQS=N*(J-1)
            DO 10 IX=JY,N
              IXJ=IQS+IX
              IHELP=J-IX
              DO 60 JX=JY,N
                IJREF=N*(JX-1)+IX
                JJX=IJREF+IHELP
   60           A(IJREF)=A(IJREF)-(A(IXJ)*A(JJX))
   10 B(IX)=B(IX)-B(J)*A(IXJ)
   70    NY=N-1
      NN=N*N
      DO 80 J=1,NY
        I1=NN-J
        I2=N-J
        I3=N
      DO 80 K=1,J
        B(I2)=B(I2)-A(I1)*B(I3)
        I1=I1-N
   80   I3=I3-1
      RETURN
      END
C
C==========================================================
C
	SUBROUTINE FINDIF(XL,YL,ICN,N,D) 
C
C
C  This subroutine computes the finite difference matrix of Eq.(8.83)
C
	IMPLICIT DOUBLEPRECISION (A-H,O-Z)                     
      DIMENSION XL(N+1),YL(N+1),D(N,N),ICN(N+1)
	N1=1
	N2=N
	NM=N
	D=0.

	DO 100 I=1,NM
	X2=XL(N1+I-1)
	Y2=YL(N1+I-1)
      X3=XL(N1+I)
	Y3=YL(N1+I)
	sl2=0.
	SL2=DSQRT((X3-X2)**2+(Y3-Y2)**2)
	IF (I.EQ.1.OR.ICN(N1+I-1).EQ.1) THEN
C	Forward finite difference
	X4=XL(N1+I+1)
 	Y4=YL(N1+I+1)
	X5=XL(N1+I+2)
	Y5=YL(N1+I+2)
	sl3=0.
	sl4=0.
	SL3=DSQRT((X3-X4)**2+(Y3-Y4)**2)
	SL4=DSQRT((X4-X5)**2+(Y4-Y5)**2)
	S1=(sL2+SL3)/2.
	S2=(sL3+SL4)/2.
	A1=-(2*S1+S2)/S1/(S1+S2)
	A2=(S1+S2)/S1/S2
	A3=-S1/S2/(S1+S2)
	D(I,I)=A1
	D(I,I+1)=A2
	D(I,I+2)=A3
	ELSEIF (I.EQ.Nm.OR.ICN(N1+I).EQ.1) THEN
C	Backward finite difference
	X0=XL(N1+I-3)
	Y0=YL(N1+I-3)
	X1=XL(N1+I-2)
	Y1=YL(N1+I-2)
	sl0=0.
	sl1=0.
	SL0=DSQRT((X0-X1)**2+(Y0-Y1)**2)
	SL1=DSQRT((X1-X2)**2+(Y1-Y2)**2)
	S1=(sL2+SL1)/2.
	S2=(sL1+SL0)/2.
	A1=(2*S1+S2)/S1/(S1+S2)
	A2=-(S1+S2)/S1/S2
	A3=S1/S2/(S1+S2)
	D(I,I-2)=A3
	D(I,I-1)=A2
	D(I,I)=A1
	ELSE
C	Central finite difference
	X1=XL(N1+I-2)
	Y1=YL(N1+I-2)
	X4=XL(N1+I+1)
	Y4=YL(N1+I+1)
	sl1=0.
	sl3=0.
	SL1=DSQRT((X1-X2)**2+(Y1-Y2)**2)
	SL3=DSQRT((X3-X4)**2+(Y3-Y4)**2)
	S1=(sL1+SL2)/2.
	S2=(sL3+SL2)/2.
	A1=-S2/S1/(S1+S2)
	A2=-(S1-S2)/S1/S2
	A3=S1/S2/(S1+S2)
	D(I,I-1)=A1
	D(I,I)=A2
	D(I,I+1)=A3
	ENDIF

  100 CONTINUE

	RETURN
	END
C	              
C==================================================================
C
      SUBROUTINE OUTPUT(XM,YM,UB,QB,XIN,YIN,U,UX,UY,UXX,UYY,UXY,N
	1  ,IN,TT)
C
C
C  This subroutine prints the results in the outputfile.
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XM(N),YM(N),UB(N),QB(N)
      DIMENSION XIN(IN),YIN(IN),U(IN),UX(IN),UY(IN),
     1          UXX(IN),UYY(IN),UXY(IN)
      dimension TT(N,N),UBT(N)

C
      WRITE(2,100)
  100 FORMAT('',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'//
     1         11X,'X',15X,'Y',12X,'U',15X,'Un',15X,'Ut'/)
     
	UBT=0.
	UbT=MATMUL(TT,UB)

      DO I=1,N
	X=XM(I)
	Y=YM(I)        
      WRITE(2,200) XM(I),YM(I),UB(I),QB(I),UBT(I)
	ENDDO
  200 FORMAT(5(2X,E14.8))
C
      WRITE (2,300)
  300 FORMAT(//,2X,'INTERNAL POINTS'//10X,'X',15X,'Y',17X,
     1 'U',16X,'UX',16X,'UY',14X,/)
      
      DO I=1,IN
      WRITE(2,400)  XIN(I),YIN(I),U(I),UX(I),UY(I)
	ENDDO

  400 FORMAT(8(2X,E15.6))
      WRITE (2,350)
  350 FORMAT(//,2X,'INTERNAL POINTS'//10X,'X',15X,'Y',14X,
     1 'UXX',13X,'UYY',14X,'UXY'/)
      DO I=1,IN
	X=XIN(I)
 	Y=YIN(I)
      WRITE(2,450)  XIN(I),YIN(I),UXX(I),UYY(I),UXY(I) 
   	ENDDO
  450 FORMAT(8(2X,E14.5))

  800 FORMAT(3(2X,E14.5))
      WRITE(2,500)
  500 FORMAT(' ',79('*'))


      RETURN
      END          
