% Script file: aemlabedata.m
%
%    This M-file prapares the data for the program AEMLABE   
%    Example 8.3ii
clear all;
fprintf('********* PLDATA ********** \n');
inputfile1=input('Enter file name :','s');
fid = fopen (inputfile1,'w');
user=input('User name:','s');
title=input('Title:','s');
%==========================================================================
% BOUNDARY DATA
%==========================================================================
nb=1;                   % number of boundaries
                             %
ns=1;                   % number of sides.
                            % A side is defined as a part of the boundary  
                            % between the corner points. Points where
                            % the boundary conditions change or points 
                            % of point supports treated as corner points.
                            % Parts of the boundary with different geometry
                            % are different sides.

% Give the side properties
%ATENSION: External boundaries are run  anticlockwise,
%          internal boundaries clockwise. 

%---------------------------------ms-----------------------------------------------          
%               1       2             3   4       5         6         7                 8        
%------------------------------------------------------------------------------------                       
%line      : 1 | No of BE | x1 | y1  |   x2   |  y2   | unused   |  unused  |    
%            
%circle   : 2 | No of BE | x0 | y0  | phi1 | phi2 | radious   | unused   | 
%          
%ellipse:  3 | No of BE | x0 | y0  | phi1 | ph2  | radious1 | radious2|            
%---------------------------------------------------------                                                              
% Give: 1 for straight line; 2 for circular arc
% x1               : x coordinate  of the starting point
% y1               : y coordinate  of the starting point
% x2               : x coordinate  of the end point
% y2               : y coordinate  of the end point

% x0               : x coordinate  of the center 
% y0               : y coordinate  of the center
% phi1, phi2       : limit angles of the circular arc
% radious1 : lenght of semiaxis 1
% radious2 : lenght of semiaxis 2
% 
% ms:= array including the side properteis
ms=zeros(ns,7);
ms=[ 3  ,100,  0   , 0    , 0  ,2*pi ,5,3       
         ];
%       
%  CALL function: boundarypoints
%                For the meanning of the output parameters see function
%                boundarypoints.m
nl=zeros(nb,1);
nl=[100];               % serial number of the last element of each boundary

[nbe,xb,yb,xl,yl,jn,kn,angle,cn,sn,length,cur]=boundarypoints(ms,ns,nl);

 % Boundary conditions for each side:
 % **********************************************************************
 % c1, c2, c3 vectors with the coefficients of the boundary conditions
 % **********************************************************************                   
      
c1=zeros(nb,1);
c2=zeros(nb,1);
c3=zeros(nb,1);

ik=0;
b=3;
a=5;
for i=1:ns
    for j=1:ms(i,2)
        ik=ik+1;
        x=xb(ik);
        y=yb(ik);
        
        c1(ik)=0;
        c2(ik)=1;
        c3(ik)=(((1+y^2)*x*b^2+x*y^2*a^2)*(2*x-y)+(b^2*x^2*y+ ...
             a^2*(1+2*x^2)*y)*(-x+2*y))/sqrt(b^4*x^2+a^4*y^2); 
    end
end
c1(1)=1;
c2(1)=0;
c3(1)=xb(1)^2-xb(1)*yb(1)+yb(1)^2;

%==========================================================================  
% GENERATION of the coorinates of the domain mesh points xin,yin where the
% solution and its derivatives  will be evaluated.
% A rectangular mesh with nrow X ncol nodal points is applied on the domain
% Then the function checkin.m detects the nodal points
% inside the domain. The function checkdist detects interior nodal points 
% at distance from the boundary smaller than a minimum prescribed distance 
% and excludes them.
%==========================================================================
nrow=11;
ncol=11;
inpoint=nrow*ncol;
ax=max(xb)-min(xb);
by=max(yb)-min(yb);
dxin=ax/(ncol+1);
dyin=by/(nrow+1);
nrow1=nrow;
ncol1=ncol;
mt=nrow1*ncol1;       
xint=zeros(1,mt);
yint=zeros(1,mt);

for i=1:nrow1
    for j=1:ncol1
        k=(i-1)*ncol1+j;
       xint(k)=min(xb)+j*dxin;
       yint(k)=min(yb)+i*dyin;
    end
end
%==========================================================================
% CALL function checkin.m to detect points inside the domain of the plate
%==========================================================================
dist=0.22;  %the minimum distance of the internal points from the boundary
[m,xin,yin]=checkin(nbe,mt,xb,yb,xint,yint,cn,sn,length,dist);
          % m  = final total number of mesh point points
%==========================================================================
% COEFFICIENTS of the Equation and RIGHT HAND SIDE of the Equation 
%==========================================================================
for i=1:m
    x=xin(i);
    y=yin(i);
    
     f(i)=7*x^2-5*x*y+5*y^2+4;
     
     
    coefa(i)=1+y^2;
    coefb(i)=x*y;
    coefc(i)=1+2*x^2;
    coefd(i)=x;
    coefe(i)=y;
    coeff(i)=1;
end

for i=1:nbe
    x=xb(i);
    y=yb(i);
    coefba(i)=1+y^2;
    coefbb(i)=x*y;
    coefbc(i)=1+2*x^2;
end

%==========================================================================
fprintf(' No of boudary elements \n');
nbe
fprintf(' No of internal  points \n');
m
%==========================================================================
% PLOT DATA
plot(xl,yl,'m+',xb,yb,'b.',xin,yin,'ro');  
%==========================================================================
% WRITE data in the inputfile
%==========================================================================
fprintf(fid,user );
fprintf(fid,'\n');
fprintf(fid,title );
fprintf(fid,'\n');
for ii=1:nbe
    fprintf(fid,'%6.8f %6.8f %0.0f\n',xl(ii),yl(ii),angle(ii));
end
for ii=1:nbe
    fprintf(fid,'%5.5f %5.5f %5.5f\n',c1(ii),c2(ii),c3(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f \n',xin(ii),yin(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f  \n',f(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f %5.5f %5.5f %5.5f %5.5f \n', ... 
        coefa(ii),coefb(ii),coefc(ii),coefd(ii),coefe(ii),coeff(ii));
end
for ii=1:nbe
    fprintf(fid,'%5.5f %5.5f %5.5f\n',coefba(ii),coefbb(ii),coefbc(ii));
end

fclose(fid);

