% Script file: aemlabedata.m
%
%    This M-file prapares the data for the program AEMLABE   
%    Example 8.4
clear all;
fprintf('********* PLDATA ********** \n');
inputfile1=input('Enter file name :','s');
fid = fopen (inputfile1,'w');
user=input('User name:','s');
title=input('Title:','s');
%==========================================================================
% BOUNDARY DATA
%==========================================================================
nb=1;                   % number of boundaries
                             %
ns=6;                   % number of sides.
                            % A side is defined as a part of the boundary  
                            % between the corner points. Points where
                            % the boundary conditions change or points 
                            % of point supports treated as corner points.
                            % Parts of the boundary with different geometry
                            % are different sides.

% Give the side properties
%ATENSION: External boundaries are run  anticlockwise,
%          internal boundaries clockwise. 

%---------------------------------ms-----------------------------------------------          
%               1       2             3   4       5         6         7                 8        
%------------------------------------------------------------------------------------                       
%line      : 1 | No of BE | x1 | y1  |   x2   |  y2   | unused   |  unused  |    
%            
%circle   : 2 | No of BE | x0 | y0  | phi1 | phi2 | radious   | unused   | 
%          
%ellipse:  3 | No of BE | x0 | y0  | phi1 | ph2  | radious1 | radious2|            
%---------------------------------------------------------                                                              
% Give: 1 for straight line; 2 for circular arc
% x1               : x coordinate  of the starting point
% y1               : y coordinate  of the starting point
% x2               : x coordinate  of the end point
% y2               : y coordinate  of the end point

% x0               : x coordinate  of the center 
% y0               : y coordinate  of the center
% phi1, phi2       : limit angles of the circular arc
% radious1 : lenght of semiaxis 1
% radious2 : lenght of semiaxis 2
% 
% ms:= array including the side properteis
ms=zeros(ns,7);
ms=[ 1 , 25 , 0 , 0 , 0.5 , 0 , 0 , 0  ;
         1 , 30 , 0.5 , 0 , 1.0 , 0.3 , 0 , 0  ;
         1 , 35 , 1 , 0.3 , 1 , 1 , 0 , 0  ;
         1 , 25 , 1 , 1 , 0.5 , 1 , 0 , 0  ;
         1 , 30 , 0.5 , 1 , 0. , 0.7 , 0 , 0  ;
         1 , 35 , 0. , 0.7 , 0. , 0 , 0 , 0  ;
         ];
%       
%  CALL function: boundarypoints
%                For the meanning of the output parameters see function
%                boundarypoints.m
nl=zeros(nb,1);
nl=[180];               % serial number of the last element of each boundary

[nbe,xb,yb,xl,yl,jn,kn,angle,cn,sn,length,cur]=boundarypoints(ms,ns,nl);

 % Boundary conditions for each side:
 % **********************************************************************
 % c1, c2, c3 vectors with the coefficients of the boundary conditions
 % **********************************************************************                   
      
c1=zeros(nb,1);
c2=zeros(nb,1);
c3=zeros(nb,1);

ik=0;
b=3;
a=5;
for i=1:ns
    for j=1:ms(i,2)
        
        ik=ik+1;
        x=xb(ik);
        y=yb(ik);
        if i == 3
            c1(ik)=0;
            c2(ik)=1;
            c3(ik)=((2*x-5*y)*(2*x+y+2)^2+(2*x-5*y)*(x-y)^2);
        else
            c1(ik)=1;
            c2(ik)=0;
            c3(ik)=x^2+y^2-5*x*y;            
        end
        
    end
end


%==========================================================================  
% GENERATION of the coorinates of the domain mesh points xin,yin where the
% solution and its derivatives  will be evaluated.
% A rectangular mesh with nrow X ncol nodal points is applied on the domain
% Then the function checkin.m detects the nodal points
% inside the domain. The function checkdist detects interior nodal points 
% at distance from the boundary smaller than a minimum prescribed distance 
% and excludes them.
%==========================================================================
nrow=11;
ncol=11;
inpoint=nrow*ncol;
ax=max(xb)-min(xb);
by=max(yb)-min(yb);
dxin=ax/(ncol+1);
dyin=by/(nrow+1);
nrow1=nrow;
ncol1=ncol;
mt=nrow1*ncol1;       
xint=zeros(1,mt);
yint=zeros(1,mt);

for i=1:nrow1
    for j=1:ncol1
        k=(i-1)*ncol1+j;
       xint(k)=min(xb)+j*dxin;
       yint(k)=min(yb)+i*dyin;
    end
end
%==========================================================================
% CALL function checkin.m to detect points inside the domain of the plate
%==========================================================================
dist=0.02;  %the minimum distance of the internal points from the boundary
[m,xin,yin]=checkin(nbe,mt,xb,yb,xint,yint,cn,sn,length,dist);
          % m  = final total number of mesh point points
%==========================================================================
% COEFFICIENTS of the Equation and RIGHT HAND SIDE of the Equation 
%==========================================================================
for i=1:m
    x=xin(i);
    y=yin(i);
    
     f(i)=-18*(x^2+y^2)+16;
     
     
    coefa(i)=(2*x+y+2)^2;
    coefb(i)=(x-y)^2;
    coefc(i)=(x+2*y+2)^2;
    coefd(i)=6*x+6*y+8;
    coefe(i)=6*x+6*y+8;
    coeff(i)=0;
end

for i=1:nbe
    x=xb(i);
    y=yb(i);
    coefba(i)=(2*x+y+2)^2;
    coefbb(i)=(x-y)^2;
    coefbc(i)=(x+2*y+2)^2;
end

%==========================================================================
fprintf(' No of boudary elements \n');
nbe
fprintf(' No of internal  points \n');
m
%==========================================================================
% PLOT DATA
plot(xl,yl,'m+',xb,yb,'b.',xin,yin,'ro');  
%==========================================================================
% WRITE data in the inputfile
%==========================================================================
fprintf(fid,user );
fprintf(fid,'\n');
fprintf(fid,title );
fprintf(fid,'\n');
for ii=1:nbe
    fprintf(fid,'%6.8f %6.8f %0.0f\n',xl(ii),yl(ii),angle(ii));
end
for ii=1:nbe
    fprintf(fid,'%5.5f %5.5f %5.5f\n',c1(ii),c2(ii),c3(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f \n',xin(ii),yin(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f  \n',f(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f %5.5f %5.5f %5.5f %5.5f \n', ... 
        coefa(ii),coefb(ii),coefc(ii),coefd(ii),coefe(ii),coeff(ii));
end
for ii=1:nbe
    fprintf(fid,'%5.5f %5.5f %5.5f\n',coefba(ii),coefbb(ii),coefbc(ii));
end

fclose(fid);

